//
//  AppDelegate.m
//  Location
//
//  Created by Alasdair Allan on 29/01/2012.
//  Copyright (c) 2012 University of Exeter. All rights reserved.
//

#import "AppDelegate.h"
#import "ViewController.h"

#include <sqlite3.h>

@implementation AppDelegate

@synthesize window = _window;
@synthesize viewController = _viewController;
@synthesize locationManager = _locationManager;
@synthesize rows = _rows;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];

    // Copy the starter location database to the Documents directory if needed
    [self copyDatabaseFromBundle];
    
    // Create geo-fence
    CLLocationCoordinate2D center = CLLocationCoordinate2DMake(50.72451, -3.52788);
    CLLocationDistance radius = 200.0; // 200m
    CLRegion *region = [[CLRegion alloc] initCircularRegionWithCenter:center radius:radius identifier:@"Exeter Apple Store"];
    
    if ( [CLLocationManager locationServicesEnabled] ) {
        self.locationManager = [[CLLocationManager alloc] init];     
        self.locationManager.delegate = self;
        if ( [CLLocationManager significantLocationChangeMonitoringAvailable] ) {
            [self.locationManager startMonitoringSignificantLocationChanges];
        } else {
            NSLog(@"Significant location change service not available.");
        }
        
        if ( [CLLocationManager regionMonitoringAvailable] ) {
            CLLocationAccuracy accuracy = 1.0;
            [self.locationManager startMonitoringForRegion:region desiredAccuracy:accuracy];
            NSLog(@"Enabling region monitoring.");
        } else {
            NSLog(@"Warning: Region monitoring not supported on this device.");
        }

    } else {
        NSLog(@"Location services not enabled.");
    }
 
    self.rows = [[NSMutableArray alloc] initWithCapacity:5];
    [self.rows insertObject:@"Lat." atIndex:0];
    [self.rows insertObject:@"Long." atIndex:1];
    [self.rows insertObject:@"Alt." atIndex:2];
    [self.rows insertObject:@"Speed" atIndex:3];
    [self.rows insertObject:@"Course" atIndex:4];
    
    self.viewController = [[ViewController alloc] initWithNibName:@"ViewController" bundle:nil];
    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
    
    UILocalNotification *localNotification = [launchOptions objectForKey:UIApplicationLaunchOptionsLocalNotificationKey];
    if (localNotification) {
        NSLog(@"Notification Body: %@",localNotification.alertBody);
        NSLog(@"%@", localNotification.userInfo);
    }
    
    application.applicationIconBadgeNumber = 0;
    [[UIApplication sharedApplication] cancelAllLocalNotifications];

    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application {
 
}

- (void)applicationDidEnterBackground:(UIApplication *)application {

}

- (void)applicationWillEnterForeground:(UIApplication *)application {

}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    application.applicationIconBadgeNumber = 0;
    [[UIApplication sharedApplication] cancelAllLocalNotifications];

}

- (void)applicationWillTerminate:(UIApplication *)application {

}

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
    NSLog(@"Location: %@", [newLocation description]);
        
    NSString *latitude = [NSString stringWithFormat:@"Lat. %f degrees", newLocation.coordinate.latitude];
    NSString *longitude = [NSString stringWithFormat:@"Long. %f degrees", newLocation.coordinate.longitude];
    NSString *altitude = [NSString stringWithFormat:@"Alt. %f m", newLocation.altitude];
    NSString *speed = [NSString stringWithFormat:@"Speed %f m/s", newLocation.speed];
    
    NSString *course = [NSString stringWithFormat:@"Course %f degrees", newLocation.course];
    
    [self.rows insertObject:latitude atIndex:0];
    [self.rows insertObject:longitude atIndex:1];
    [self.rows insertObject:altitude atIndex:2];
    [self.rows insertObject:speed atIndex:3];
    [self.rows insertObject:course atIndex:4];
    
    [self addLocationToDatabase:newLocation];
    
    [self.viewController.tableView reloadData];

}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    NSLog(@"Error: %@", [error description]);
}

- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region {
    
    UILocalNotification *notification = [[UILocalNotification alloc] init];
    notification.fireDate = [NSDate date];
    notification.timeZone = [NSTimeZone defaultTimeZone];
    
    notification.alertBody = @"Entered Region";
    notification.soundName = UILocalNotificationDefaultSoundName;
    notification.applicationIconBadgeNumber = 1;
    
    [[UIApplication sharedApplication] scheduleLocalNotification:notification];

}

- (void)locationManager:(CLLocationManager *)manager didExitRegion:(CLRegion *)region {
    
    UILocalNotification *notification = [[UILocalNotification alloc] init];
    notification.fireDate = [NSDate date];
    notification.timeZone = [NSTimeZone defaultTimeZone];
    
    notification.alertBody = @"Exited Region";
    notification.soundName = UILocalNotificationDefaultSoundName;
    notification.applicationIconBadgeNumber = 1;
    
    [[UIApplication sharedApplication] scheduleLocalNotification:notification];

}



- (NSString *)databaseLocation {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsPath = [paths objectAtIndex:0];
    NSString *filePath = [documentsPath stringByAppendingPathComponent:@"location.db"];
    return filePath;
}

- (void)copyDatabaseFromBundle {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *filePath = [self databaseLocation];
    
    // We don't have a database, maybe this is the first time we've been run?
    if ( ![fileManager fileExistsAtPath:filePath] ) {
        NSString *bundlePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"location.db"];
        [fileManager copyItemAtPath:bundlePath toPath:filePath error:nil];
    }
    
}

-(void) addLocationToDatabase:(CLLocation *)newLocation {
    
    sqlite3 *database;
    
    if(sqlite3_open([[self databaseLocation] UTF8String], &database) == SQLITE_OK) {
        const char *sqlStatement = "INSERT INTO location (timestamp, latitude, longitude, altitude, horizontalAccuracy, verticalAccuracy, speed, course) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
        sqlite3_stmt *compiledStatement;
        if(sqlite3_prepare_v2(database, sqlStatement, -1, &compiledStatement, NULL) == SQLITE_OK)    {
            
            sqlite3_bind_text(compiledStatement, 1, [[newLocation.timestamp description] UTF8String], -1, SQLITE_TRANSIENT);
            sqlite3_bind_double(compiledStatement, 2, newLocation.coordinate.latitude);
            sqlite3_bind_double(compiledStatement, 3, newLocation.coordinate.longitude);
            sqlite3_bind_double(compiledStatement, 4, newLocation.altitude);
            sqlite3_bind_double(compiledStatement, 5, newLocation.horizontalAccuracy);
            sqlite3_bind_double(compiledStatement, 6, newLocation.verticalAccuracy);
            sqlite3_bind_double(compiledStatement, 7, newLocation.speed);
            sqlite3_bind_double(compiledStatement, 8, newLocation.course);
            
        }
        if(sqlite3_step(compiledStatement) != SQLITE_DONE ) {
            NSLog( @"Error: %s", sqlite3_errmsg(database) );
        } else {
            NSLog( @"Insert into row id = %d", (int)sqlite3_last_insert_rowid(database));
        }
        sqlite3_finalize(compiledStatement);
    }
    sqlite3_close(database);
    
}


- (CLLocation *)readLocationFromDatabaseWithIndex:(int)index {
    
    CLLocation *location = nil;
    sqlite3 *database;
    
    if(sqlite3_open([[self databaseLocation] UTF8String], &database) == SQLITE_OK) {
        const char *sqlStatement = [[NSString stringWithFormat:@"SELECT * FROM location WHERE id = '%d'", index] cStringUsingEncoding:NSASCIIStringEncoding];
        sqlite3_stmt *compiledStatement;
        if(sqlite3_prepare_v2(database, sqlStatement,
                              -1, &compiledStatement, NULL) == SQLITE_OK) {
            if(sqlite3_step(compiledStatement) == SQLITE_ROW) {
                
                NSString *timestamp = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 1)];
                
                // Convert string to date object
                NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
                [dateFormat setDateFormat:@"%Y-%m-%d %H:%M:%S %z"];
                NSDate *date = [dateFormat dateFromString:timestamp];
                
                float latitude = sqlite3_column_double(compiledStatement, 2);
                float longitude = sqlite3_column_double(compiledStatement, 3);
                float altitude = sqlite3_column_double(compiledStatement, 4);
                float horizontalAccuracy = sqlite3_column_double(compiledStatement, 5);
                float verticalAccuracy = sqlite3_column_double(compiledStatement, 6);
                float speed = sqlite3_column_double(compiledStatement, 7);
                float course = sqlite3_column_double(compiledStatement, 8);
                
                CLLocationCoordinate2D coordinate = CLLocationCoordinate2DMake(latitude, longitude);
                location = [[CLLocation alloc] initWithCoordinate:coordinate altitude:(CLLocationDistance)altitude horizontalAccuracy:(CLLocationAccuracy)horizontalAccuracy verticalAccuracy:(CLLocationAccuracy)verticalAccuracy course:(CLLocationDirection)course speed:(CLLocationSpeed)speed timestamp:date];
            }
        }
        sqlite3_finalize(compiledStatement);
    }
    sqlite3_close(database);
    return location;
}

- (int)getSizeOfDatabase {
    
    int count = 0;
    sqlite3 *database;
    
    if (sqlite3_open([[self databaseLocation] UTF8String], &database) == SQLITE_OK)
    {
        const char* sqlStatement = "SELECT COUNT(*) FROM location";
        sqlite3_stmt* statement;
        
        if( sqlite3_prepare_v2(database, sqlStatement, -1, &statement, NULL) == SQLITE_OK )
        {
            if( sqlite3_step(statement) == SQLITE_ROW )
                count  = sqlite3_column_int(statement, 0);
        }
        else
        {
            NSLog( @"Failed from sqlite3_prepare_v2. Error is:  %s", sqlite3_errmsg(database) );
        }
        
        // Finalize and close database.
        sqlite3_finalize(statement);
        sqlite3_close(database);
    }
    
    return count;
}

@end
